
/*
 * defines.h
 *
 *  Author: razvan
 */ 

#ifndef _DEFINES_H
#define _DEFINES_H

    #define F_CPU                           16000000

    #define ZERO                            r1
    #define ONE                             r0

    #define SWITCH_TOGGLE_CURRENT           r20
    #define SWITCH_TOGGLE_MASK              r21
    #define TEMP_REGISTER_ONE               r22
    #define TEMP_REGISTER_TWO               r23
    
    #define ADC_AUTO_ZERO_READING_L         r2
    #define ADC_AUTO_ZERO_READING_H         r3
    #define ADC_AUTO_ZERO_READING           ADC_AUTO_ZERO_READING_H, ADC_AUTO_ZERO_READING_L
    
    #define SLOPE_COUNT_VRP_L               r4
    #define SLOPE_COUNT_VRP_H               r5
    #define SLOPE_COUNT_VRP                 SLOPE_COUNT_VRP_H, SLOPE_COUNT_VRP_L

    #define SLOPE_COUNT_SWITCH_VRP_VRN_L    r6
    #define SLOPE_COUNT_SWITCH_VRP_VRN_H    r7
    #define SLOPE_COUNT_SWITCH_VRP_VRN      SLOPE_COUNT_SWITCH_VRP_VRN_H, SLOPE_COUNT_SWITCH_VRP_VRN_L
    
    #define SLOPE_COUNT_VRN_L               r8
    #define SLOPE_COUNT_VRN_H               r9
    #define SLOPE_COUNT_VRN                 SLOPE_COUNT_VRN_H, SLOPE_COUNT_VRN_L

    #define SLOPE_COUNT_SWITCH_VRN_VRP_L    r10
    #define SLOPE_COUNT_SWITCH_VRN_VRP_H    r11
    #define SLOPE_COUNT_SWITCH_VRN_VRP      SLOPE_COUNT_SWITCH_VRN_VRP_H, SLOPE_COUNT_SWITCH_VRN_VRP_L
    
    #define SLOPE_COUNT_PRERUNDOWN          r12
    #define SLOPE_COUNT_RUNDOWN_VRP         r13
    #define SLOPE_COUNT_RUNDOWN_VRN         r14
    #define SLOPE_COUNT_RUNDOWN_VRP_SMALL   r15

    #define ADC_RUNDOWN_READING_L           r16
    #define ADC_RUNDOWN_READING_H           r17
    #define ADC_RUNDOWN_READING             ADC_RUNDOWN_READING_H, ADC_RUNDOWN_READING_L


    #define SWITCH_NONE                     0
    #define SWITCH_VIN                      _BV(PORTC0)
    #define SWITCH_VREF_N                   _BV(PORTC1)
    #define SWITCH_VREF_P                   _BV(PORTC2)
    #define SWITCH_RESET                    _BV(PORTC3)
    #define SWITCH_GND                      _BV(PORTC4)
    #define SWITCH_SH                       _BV(PORTC5)
    #define SWITCH_VREF_SMALL               _BV(PORTC6)
    #define SWITCH_PORT                     PORTC
    
    // Status indication
    #define STATUS_NONE                     0x00
    #define STATUS_ERROR                    _BV(PORTB0)
    #define STATUS_PORT                     PORTB

    // Input for comparators 
    #define INPUT_ZERO_THRESHOLD            PORTD2
    #define INPUT_NONE                      0
    // We need a bit on the "input" port for a custom event
    #define INPUT_MASK                      ((1 << INPUT_ZERO_THRESHOLD) | (1 << INPUT_ZERO_THRESHOLD))
    #define INPUT_CHECK_REGISTER            PIND

    #define UART_TX                         _BV(PORTD1)
    #define UART_RX                         _BV(PORTD0)
    #define BAUD_RATE                       500000
    #define BRR_REG                         (F_CPU/(16 * (BAUD_RATE)))      
      
    #define us_to_cycles(value) ((F_CPU / 1000000) * (value))
    #define ms_to_cycles(value) ((F_CPU / 1000) * (value))

    // The number of clock cycles we integrate before we check if we passed zero
    #define INTEGRATE_TP                    us_to_cycles(1) // Clock cycles     
    // The number of clock cycles we do a PRERUNDOWN cycle before we check if we passed zero
    #define PRERUNDOWN_TP                   us_to_cycles(1) // Clock cycles
    // The number of clock cycles we do a RUNDOWN cycle before we check if we passed zero
    #define RUNDOWN_TP                      5 // Clock cycles

    // In the runup counters we can hold at most 65.5 milliseconds if INTEGRATE_TP is 1 microsecond
    #define RUNUP_TIME_IN_MS                16 // Milliseconds

    // Maximum time we can spend in RUNUP without overflowing counters in worst case scenarios
    //  for INTEGRATE_TP = 1us it is 65535us total ~65ms
    //  for INTEGRATE_TP = 4us it is 4*65.5ms -> 262ms

    // Maximum integration time period we can have is 262ms due to maximum OCR1A size of 16 bits
    //  250 ticks per ms due to a prescale of 64 and 16Mhz clock * 262ms -> 65500
    // To be safe we clamp at 200
    #define MAX_RUNUP_IN_MS                 ((0xFFFF * (INTEGRATE_TP / us_to_cycles(1))) / 1000)
    #define MAX_SUPPORTED_RUNDOWN_LOOPS     (0xFF / ((INTEGRATE_TP) / (RUNDOWN_TP)))
    #define MAX_SUPPORTED_PRERUNDOWN_LOOPS  (0xFF / ((INTEGRATE_TP) / (PRERUNDOWN_TP)))

    #if RUNUP_TIME_IN_MS > 200
        #error "Maximum integration time exceeeded. At most it can be 200ms"    
    #endif
    #if RUNUP_TIME_IN_MS > MAX_RUNUP_IN_MS
        #error "Maximum runup time exceeded. Consider increasing INTEGRATE_TP, or decreasing RUNUP_TIME_IN_MS"
    #endif
    #if MAX_SUPPORTED_PRERUNDOWN_LOOPS < 4
        #error "Counter for PRERUNDOWN will overflow. Consider decreasing INTEGRATE_TP or increasing PRERUNDOWN_TP"
    #endif
    #if MAX_SUPPORTED_RUNDOWN_LOOPS < 4
        #error "Counter for RUNDOWN will overflow. Consider decreasing INTEGRATE_TP or increasing RUNDOWN_TP"
    #endif

#endif