
/*
 * delay_macros.s
 *
 *  Author: razvan
 */ 
#ifndef _DELAY_S
#define _DELAY_S

#include "defines.h"
#include <avr/io.h>


DELAY_FUNC_SMALL:
1:  dec TEMP_REGISTER_ONE
    brne 1b
ret

DELAY_FUNC_LARGE:
1:  subi TEMP_REGISTER_ONE, 1
    sbc  TEMP_REGISTER_TWO, R1
    or   TEMP_REGISTER_ONE, TEMP_REGISTER_TWO
    brne 1b
    nop
ret


.macro DELAY_NOP count:req
    .if \count > 0 
        nop
    .elseif \count < 0
        .error "Delay cannot be negative"
    .endif
    .if (\count-1) > 0
        DELAY_NOP (\count-1)
    .endif
.endm

// Generate delays of x clock cycles
.macro DELAY clockCycles:req
    .if \clockCycles > (0xFF*3 + 6)
        .if \clockCycles > (0xFFFF*5 + 8)
            .error "Maximum delay exceeded. 458752 maximum cycles delay."
            .exitm
        .endif
        .set LOOP_CNT, ((\clockCycles-8)/5)
        DELAY (\clockCycles - (LOOP_CNT*5+8))
        ldi TEMP_REGISTER_ONE, (LOOP_CNT >> 8)
        ldi TEMP_REGISTER_TWO, (LOOP_CNT & 0xFF)
        rcall DELAY_FUNC_LARGE
    .else
        .if \clockCycles < 3
            DELAY_NOP \clockCycles
        .elseif \clockCycles >= 9
            .set LOOP, ((\clockCycles - 6)/3)
            DELAY_NOP (\clockCycles - (LOOP * 3 + 6))
            ldi TEMP_REGISTER_ONE, LOOP
            rcall DELAY_FUNC_SMALL
        .else
            DELAY_NOP \clockCycles % 3
            ldi TEMP_REGISTER_ONE, (\clockCycles / 3)
  delay_\@: dec TEMP_REGISTER_ONE
            brne delay_\@
        .endif
    .endif
.endm

.macro DELAY_MICROS micros:req
    DELAY ((F_CPU/1000000) * \micros)
.endm

.macro DELAY_MS ms:req
    DELAY_MICROS (\ms * 1000)
.endm

#endif