
/*
 * macros.S
 *
 *  Author: razvan
 */ 
#ifndef _HELPERS_S
#define _HELPERS_S

#include <avr/io.h>
#include "defines.h"
#include "delay.s"
 
#define IS_ZERO(register, bit) register, bit, 0
#define IS_ONE(register, bit) register, bit, 1

.macro PUSHA
    .irp    param,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,28,29
        push \param
    .endr
.endm

.macro POPA
    .irp    param,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,28,29
        pop \param
    .endr
.endm

.macro LOOP_IF targetIO:req, bit:req, condition:req, label:req
    .if \condition == 1
        sbic _SFR_IO_ADDR(\targetIO), \bit
    .else
        sbis _SFR_IO_ADDR(\targetIO), \bit
    .endif
    rjmp \label
    nop
.endm

.macro JUMP_IF targetIO:req, bit:req, condition:req, label:req
    .if _SFR_IO_ADDR(\targetIO) < 0x20
        .if \condition == 1
            sbic _SFR_IO_ADDR(\targetIO), \bit
        .else
            sbis _SFR_IO_ADDR(\targetIO), \bit
        .endif
        jmp \label
        DELAY 1
    .else
        lds TEMP_REGISTER_ONE, _SFR_IO_ADDR(\targetIO)
        .if \condition == 1
            sbrc TEMP_REGISTER_ONE, \bit
        .else
            sbrs TEMP_REGISTER_ONE, _SFR_IO_ADDR(\targetIO), \bit
        .endif
        jmp \label
        DELAY 1
    .endif
.endm

.macro LOOP_IF_NEGATIVE label:req
    LOOP_IF IS_ZERO(INPUT_CHECK_REGISTER, INPUT_ZERO_THRESHOLD), \label
.endm

.macro LOOP_IF_POSITIVE label:req
    LOOP_IF IS_ONE(INPUT_CHECK_REGISTER, INPUT_ZERO_THRESHOLD), \label
.endm

.macro SWITCH switchState:req
    ldi TEMP_REGISTER_ONE,  \switchState
    out _SFR_IO_ADDR(SWITCH_PORT), TEMP_REGISTER_ONE
.endm

.macro SWITCH_TOGGLE currentState:req, switchMask:req, additionalFlags:req
    ldi TEMP_REGISTER_ONE, \additionalFlags
    or TEMP_REGISTER_ONE, \currentState
    eor \currentState, \switchMask
    out _SFR_IO_ADDR(SWITCH_PORT), TEMP_REGISTER_ONE
.endm

.macro STATUS status:req
    ldi TEMP_REGISTER_ONE,  (1 << \status)
    out _SFR_IO_ADDR(STATUS_PORT), TEMP_REGISTER_ONE
.endm

.macro UART_SEND_REG reg:req, reg2
1:  lds TEMP_REGISTER_ONE, UCSR0A
    sbrc TEMP_REGISTER_ONE, UDRE0
    rjmp 1b
    sts UDR0, \reg
    // Second register if we have it
    .ifnb \reg2
    1:  lds TEMP_REGISTER_ONE, UCSR0A
        sbrc TEMP_REGISTER_ONE, UDRE0
        rjmp 1b
        sts UDR0, \reg2
    .endif

.endm

.macro UART_SEND_CHAR char:req
    ldi TEMP_REGISTER_TWO, \char
    UART_SEND_REG TEMP_REGISTER_TWO
.endm

.macro INCR high:req, low
    .ifnb \low
        add \low, ONE
        adc \high, ZERO
    .else
        add \high, ONE
    .endif
// I can't figure out how to conditionally include this if we are in DEBUG, nothing works
//     for now disable
#    rcall OVER_UNDER_CHECK
.endm

.macro DECR high:req, low
    .ifnb \low
        sub \low, ONE
        sbc \high, ZERO
    .else
        sub \high, ONE
    .endif
// I can't figure out how to conditionally include this if we are in DEBUG, nothing works
//     for now disable
#    rcall OVER_UNDER_CHECK
.endm

OVER_UNDER_CHECK:
    brcc 1f
    STATUS STATUS_ERROR
    jmp exit
1:  ret

#endif