#include <avr/io.h>

#include "defines.h"

extern void MEASURE(uint16_t integrationTime);

extern void START_ADC();

int main(){
    // Disable all interrupts, we are not using them
    asm ("cli");
    
    START_ADC();

    // UART init
    UBRR0 = BRR_REG;
    UCSR0B = (1 << RXEN0) | (1 << TXEN0);
    
    DDRD &= ~(UART_RX | INPUT_MASK);
    // Switch states are output
    DDRC = UART_TX | SWITCH_VIN | SWITCH_VREF_N | SWITCH_RESET | SWITCH_VREF_P | SWITCH_VREF_SMALL | SWITCH_SH;
    // Set the status port direction
    DDRB = STATUS_ERROR;
    // Clear the status
    STATUS_PORT = STATUS_NONE;

    // Set timer timeout
    // 64 timer prescale, 1000 clocks / ms
    uint16_t integrationTimeInMs =  (uint16_t)((uint32_t)(F_CPU / (64000)) * RUNUP_TIME_IN_MS - 1);
    for(;;) {
        MEASURE(integrationTimeInMs);
		// If we exit the MEASURE method it means we got a command from UART
        // Here we should check what command we got from UART, and start processing commands
    }
    return 0;
}
